/*
	Tooltip Application
	Version 			: 0.1.1
	Developer 		: Joseph Smith
	Release Date	: 26/06/2014
*/

var tooltipApplication = (function($)
{

	//create variable groups
	var elmentSelector    		= '.hasTooltip',
			elementDataSelector   = 'tooltip-selector',
			tooltipDirection      = 'right',
			tooltipAnimation      = 'fadeIn',
			tooltipClass          = 'tooltip',
			tooltipSpacing				=	 20,
			tooltipElement        = '<div>';

	//Building tooltips
	var buildTooltips = function()
	{
		var $elements  		 = $(elmentSelector),
				tooltipInteger = 1;

		//foreach tooltip selector
		$elements.each(function()
		{
			var $ele       		 = $(this),
					$tooltip       = $(tooltipElement, {id: 'tooltip-' + tooltipInteger, class: tooltipClass}),
					eleOffset      = $ele.offset(),
					eleTop         = eleOffset.top,
					eleLeft        = eleOffset.left,
					eleWidth       = $ele.outerWidth(),
					eleHeight      = $ele.outerHeight(),
					direction      = '',
					animation      = '',
					spacing        = '',
					topPosition    = 'auto',
					bottomPosition = 'auto',
					leftPosition   = 'auto',
					rightPosition  = 'auto',
					animateDirection   = '';

			//get data information to overwrite the direction settings
			if ($ele.attr('data-tooltip-direction'))
			{
				direction = $ele.data('tooltip-direction');
			}
			else
			{
				direction = tooltipDirection;
			}

			//get data information to overwrite the animation settings
			if($ele.attr('data-tooltip-animation'))
			{
				animation = $ele.data('tooltip-animation');
			}
			else
			{
				animation = tooltipAnimation;
			}

			//get data information to overwrite the spacing settings
			if($ele.attr('data-tooltip-spacing'))
			{
				spacing = $ele.data('tooltip-spacing');
			}
			else
			{
				spacing = tooltipSpacing;
			}

			//add the data-tooltip selector to the element
			$ele.attr('data-' + elementDataSelector, 'tooltip-' + tooltipInteger);
			//add text to the tooltip from the title attribute
			$tooltip.text($ele.attr('title'));
			//append the tooltip to body
			$tooltip.appendTo('body');

			//collect infomation
			var tooltipHeight = $tooltip.outerHeight(),
					tooltipWidth  = $tooltip.outerWidth();

			//add the correct positioning
			if(direction === 'right')
			{
				leftPosition = eleLeft + eleWidth + spacing + 'px';
				topPosition = (eleTop + (eleHeight / 2)) - (tooltipHeight / 2) + 'px';
				animateDirection = 'Left';
			}
			else if(direction === 'bottom')
			{
				leftPosition = (eleLeft + (eleWidth / 2)) - (tooltipWidth / 2) + 'px';
				topPosition = eleTop + eleHeight + spacing + 'px';
				animateDirection = 'Down';
			}
			else if(direction === 'left')
			{
				leftPosition = eleLeft - tooltipWidth - spacing + 'px';
				topPosition = (eleTop + (eleHeight / 2)) - (tooltipHeight / 2) + 'px';
				animateDirection = 'Right';
			}
			else if(direction === 'top')
			{
				leftPosition = (eleLeft + (eleWidth / 2)) - (tooltipWidth / 2) + 'px';
				topPosition = eleTop - tooltipHeight - spacing + 'px';
				animateDirection = 'Up';
			}

			//Adding the correct class for the animation
			$tooltip.addClass(animation + animateDirection);
			$tooltip.addClass('arrow-' + direction);

			//add the styles to the tooltip
			$tooltip.css({
				'left': leftPosition,
				'top' : topPosition
			});

			//add to the tooltip integer
			tooltipInteger = tooltipInteger + 1;
		});
	};

	//Mouse over event for the element
	var showTooltip = function()
	{
		var $ele      = $(this),
				tooltipID = $ele.data(elementDataSelector);

		$('#' + tooltipID).show();
	};

	//Mouse out event for the element
	var hideTooltip = function()
	{
		var $ele      = $(this),
				tooltipID = $ele.data(elementDataSelector);

		$('#' + tooltipID).hide();
	};


	//fixing the position on resizing of the browser
	var repositionTooltips = function()
	{
		var $elements = $(elmentSelector);
		//Foreach tooltip
		$elements.each(function()
		{
			var $ele       		  = $(this),
					tooltipSelector = $ele.data(elementDataSelector),
					$tooltip 				= $('#' + tooltipSelector),
					eleOffset       = $ele.offset(),
					eleTop          = eleOffset.top,
					eleLeft         = eleOffset.left,
					eleWidth        = $ele.outerWidth(),
					eleHeight       = $ele.outerHeight(),
					direction      = '',
					spacing        = '',
					topPosition     = 'auto',
					bottomPosition  = 'auto',
					leftPosition    = 'auto',
					rightPosition   = 'auto';

			//get data information to overwrite the direction settings
			if ($ele.attr('data-tooltip-direction'))
			{
				direction = $ele.data('tooltip-direction');
			}
			else
			{
				direction = tooltipDirection;
			}

			//get data information to overwrite the spacing settings
			if($ele.attr('data-tooltip-spacing'))
			{
				spacing = $ele.data('tooltip-spacing');
			}
			else
			{
				spacing = tooltipSpacing;
			}

			//collect infomation
			var tooltipHeight = $tooltip.outerHeight(),
					tooltipWidth  = $tooltip.outerWidth();

			//add the correct positioning
			if(direction === 'right')
			{
				leftPosition = eleLeft + eleWidth + spacing + 'px';
				topPosition = (eleTop + (eleHeight / 2)) - (tooltipHeight / 2) + 'px';
			}
			else if(direction === 'bottom')
			{
				leftPosition = (eleLeft + (eleWidth / 2)) - (tooltipWidth / 2) + 'px';
				topPosition = eleTop + eleHeight + spacing + 'px';
			}
			else if(direction === 'left')
			{
				leftPosition = eleLeft - tooltipWidth - spacing + 'px';
				topPosition = (eleTop + (eleHeight / 2)) - (tooltipHeight / 2) + 'px';
			}
			else if(direction === 'top')
			{
				leftPosition = (eleLeft + (eleWidth / 2)) - (tooltipWidth / 2) + 'px';
				topPosition = eleTop - tooltipHeight - spacing + 'px';
			}

			//add the styles to the tooltip
			$tooltip.css({
				'left': leftPosition,
				'top' : topPosition
			});
		});
	};


	//Start the ignition function
	var ignition = function()
	{
		var $document = $(document);
		var $window = $(window);
		buildTooltips();
		$document.on('mouseover', elmentSelector, showTooltip);
		$document.on('mouseout', elmentSelector, hideTooltip);
		$window.on('resize', repositionTooltips);
	};

	//Create the jQuery application methods
	return {
		createTooltips: ignition,
		redraw: repositionTooltips
	};
})(jQuery);